import dataiku
import dash
from dash import dcc, html, dash_table
from dash.dependencies import Input, Output, State, ALL
import sklearn
import io
import pickle
from flask import request
from main_functions import *
from layout_components_style import get_main_style, get_left_panel_style, get_right_panel_style,get_tab_style, get_tab_selected_style,get_dropdown_style, get_header_h2_style, get_subheader_style,get_description_style, get_table_styles, get_button_style, get_card_style,get_selected_card_style, get_enter_box_style, get_tooltip_style, get_question_mark_style, get_visible_tab_style
import dash_bootstrap_components as dbc

project = dataiku.api_client().get_default_project()
table_styles = get_table_styles()
button_style=get_button_style()
button_style_space = {**button_style,  "margin-right": "7px"}
card_style = get_card_style()
selected_card_style = get_selected_card_style()
enter_box_style = get_enter_box_style()
tooltip_style = get_tooltip_style() 
question_mark_style = get_question_mark_style()
header_h2_style = get_header_h2_style()
subheader_style = get_subheader_style()
description_style = get_description_style()
main_style = get_main_style()
left_panel_style = get_left_panel_style()
right_panel_style = get_right_panel_style()
dropdown_style = get_dropdown_style()
hidden_style = {'display': 'none'}
visible_style = {}
visible_tab_style = get_visible_tab_style()
card_margin =  {'margin': '4px 0', 'padding': '1'}
tab_style = get_tab_style()
tab_selected_style = get_tab_selected_style()

app.layout = html.Div([
    # Page Title

    # Main container for left and right panels
    html.Div([
        # Left Panel: Tabs for the left panel
        html.Div([
            dcc.Tabs(id='left-panel-tabs', value='tab-1', children=[
                dcc.Tab(label='', value='tab-1', children=[
                    # Left Panel Content for 'Segmentation Session'
                    html.Div([
                          html.H2(['Select a Session',
                                html.Span('?', id='tooltip-select-session', style=question_mark_style)
                            ], style=header_h2_style),
                            dbc.Tooltip(
                                "Choose one of the existing active session to view and update segmentation results. Make sure that the updated dataset exists in the dataiku flow with the original dataset name as shown in the card.",
                                target="tooltip-select-session",  
                                placement="right",
                                style = tooltip_style
                            ),
                        dcc.Input(
                            id='search-bar',
                            type='text',
                            placeholder='Search sessions...',
                            style=enter_box_style
                        ),
                        html.Div(
                            id='card-container',
                            className='card-container',
                            style={
                                'height': '900px',
                                'overflow-y': 'scroll',
                                'border': '1px solid #ddd',
                                'padding': '10px',
                                'background-color': 'white'
                            }
                        )
                    ])
                ], style=hidden_style, 
                   selected_style=visible_tab_style,
                       ),
                
                dcc.Tab(label='', value='tab-2', children=[
                    # Left Panel Content for 'Results'
                    html.Div([                        
                        
                    # Export and Save
                    html.H2(["Segmentation Processing",  html.Span('?', id='tooltip-segments-process-update', style=question_mark_style),
                        ], style=header_h2_style),
                        dbc.Tooltip(
                                "If you want to SAVE this updated segmentation session you must provide a name and segmentation description. These information are particularly useful metadata for future updates.",
                                target="tooltip-segments-process-update",  
                                placement="right",
                                style = tooltip_style
                            ),
                             dcc.Textarea(
                            id='metadata-description',
                            placeholder='Enter a segmentation description..',
                            style= description_style
                                        ),
                    html.Div([
                        html.Button("Save", id="save-icon", style=button_style_space),
                        dbc.Tooltip("SAVE button will create a csv file with the new Segmentation Results in the output_data_folder and a new record in the metadata_dataset with your updated session information.",
                                            target="save-icon",  
                                            placement="bottom",
                                            style=tooltip_style
                                        ),
                        html.Button("Export", id="export-icon", style=button_style),
                        dbc.Tooltip("EXPORT button will download a csv file with the segmentation resuts locally.",
                                            target="export-icon",  
                                            placement="bottom",
                                            style=tooltip_style
                                        ),
                        html.Span('?', id='tooltip-buttons-update', style=question_mark_style)
                    ]),
                    dbc.Tooltip(
                                "Hover over the buttons to see functionality details.",
                                target="tooltip-buttons-update",  
                                placement="right",
                                style = tooltip_style
                            ),
                      
                        
                        # Placeholder for download functionality
                        dcc.Download(id="download-dataframe-csv"),
                        html.Div(id='save-output'),
                        dcc.Store(id='description-dictionary'),
                        dcc.Store(id='filter-dictionary-store'),
                        dcc.Store(id='kmeans-pipeline-store'),
                        dcc.Store(id='rulebased-bin-bounds'),
                        dcc.Store(id='rulebased-weights'),
                    ])
                ],style=hidden_style, 
                   selected_style=visible_tab_style,
                       )
            ])
        ], style=left_panel_style),

        # Right Panel: Tabs for the right panel
        html.Div([
            dcc.Tabs(id='right-panel-tabs', value='tab-1', children=[
                dcc.Tab(label='1. Segmentation Session ➔', value='tab-1', 
                        style=tab_style, selected_style=tab_selected_style, 
                        children=[
                    # Right Panel Content for 'Segmentation Session'
                    html.Div([
                        
                        html.Div(id='selected-session-name', style={'font-weight': 'bold', 'margin-bottom': '10px'}),
                        html.Div(id='file-content-container', 
                                 children=[
                                    # html.H2('Active Version Results Display', style=header_h2_style),
                                                            dash_table.DataTable(
                                                            id='file-content-table',
                                                           columns=[],
                                                            data=[],
                                                            filter_action='native',
                                                            sort_action='native',
                                                            page_size=5,
                                                            **table_styles
                                                                ),
                        
                        html.Div(id='file-content-error', style={'color': 'black', 'margin-top': '10px'})
                        ]),

                       # html.Div(id='file-content-error', style={'color': 'black', 'margin-top': '10px'}),
                        html.Div([
                            html.Button('Update', id='update-button', style={**button_style_space, 'display': 'none'}),
                                dbc.Tooltip("Once you select the UPDATE button, the same filtering, segmentation method, and feature selection displayed on the selected card will be applied to the updated original dataset. The updated version won't be saved unless you select SAVE in the Insights tab.",
                                            target="update-button", 
                                            placement="bottom",
                                            style = tooltip_style
                                           ),
                                dbc.Tooltip(
                                "The table above shows the results of the last segmentation version and dataset. Once 'UPDATE' is selected, a new table and a Sankey graph below will display the updated segmentation results, allowing you to compare the changes in account segmentations. Note that table includes only the records that there was a cluster change. Sankey graph and Results in the next tab process the entire updated dataset.",  
                                target="tooltip-update-button",  
                                placement="right",  
                                style = tooltip_style
                                ),
                            html.Span('?', id='tooltip-update-button', style={**question_mark_style, 'display': 'none'}),
                        dbc.Tooltip(
                                "The table above shows the results of the last segmentation version and dataset. Once 'UPDATE' is selected, a new table and a Sankey graph below will display the updated segmentation results, allowing you to compare the changes in account segmentations. Note that table includes only the records that there was a cluster change. Sankey graph and Results in the next tab process the entire updated dataset.",  
                                target="tooltip-update-button",  
                                placement="right",  
                                style = tooltip_style
                                ),
                             ], style={'display': 'flex', 'align-items': 'center'}),
                        html.Div(id='update-message', style={'color': 'black', 'margin-top': '10px'}),
                        
                        html.Div([
 
                               dcc.Loading(
                                        id="loading-data-upload",
                                        type="circle",
                                        children=[ 
                                            html.Div([
                                                html.Div(
                                                    dash_table.DataTable(
                                                        id='output_table',
                                                        columns=[],
                                                        data=[],
                                                        filter_action='native',
                                                        sort_action='native',
                                                        page_size=10,
                                                        **table_styles
                                                    ),
                                                    id='table-container',  # Wrapper for the table to control visibility
                                                    style={'display': 'none'}  # Initially hidden
                                                ),
                                                html.Div(
                                                    id="no-data-message", 
                                                    style={'textAlign': 'center', 'fontSize': 12, 'color': 'gray'}  # Message container
                                                )
                                            ]),
                                        ],
                                       ),                            
                         html.Div([
                                dcc.Graph(id='sankey-diagram')
                            ], style={'width': '69%', 'display': 'inline-block', 'padding-left': '10px'}),
                        ], style={'display': 'flex', 'justify-content': 'space-between'}),

                        # Storage elements for state management
                        dcc.Store(id='selected-features-store'),
                        dcc.Store(id='session-name-store'),
                        dcc.Store(id='version-store'),
                        dcc.Store(id='original-dataset-store'),
                        dcc.Store(id='method-store'),
                        dcc.Store(id='specifications-store'),
                        dcc.Store(id='remap-dict-store'),
                        dcc.Store(id='full-data-store')
                    ])
                ]),
                dcc.Tab(label='2. Insights', value='tab-2', 
                        style=tab_style, selected_style=tab_selected_style,children=[
                        html.Div([
                            # Right Panel: Segmentation Analysis and Feature Distribution Analysis
                            html.Div([
                                html.H2(['Segmentation Insights',
                                html.Span('?', id='tooltip-segmentation-insights-update', style=question_mark_style)
                            ], style=header_h2_style),
                            dbc.Tooltip(
                                "Once you select and UPDATE a segmentation session, the graphs below will support the explainability of your updated results.",
                                target="tooltip-segmentation-insights-update",  
                                placement="right",
                                style = tooltip_style
                            ),
                               html.Div([
                            html.Div([
                                html.H3("Number of Records in Each Segment", style = subheader_style),
                                dcc.Graph(id='cluster-histogram'),
                            ], style={'width': '49%', 'display': 'inline-block'}),
                            html.Div([
                                html.H3(['Variable Importance',
                                html.Span('?', id='tooltip-variable-importance', style=question_mark_style)
                            ], style=subheader_style),
                            dbc.Tooltip(
                                "For machine learning, variable importance identifies which features most influence the formation of segments. In rule-based segmentation, feature weights directly control how much each feature contributes to the segmentation outcome. ",
                                target="tooltip-variable-importance",  
                                placement="right",
                                style = tooltip_style
                            ),
                                dcc.Graph(id='importance-histogram'),
                            ], style={'width': '49%', 'display': 'inline-block', 'float': 'right'}),
                        ]),

                                html.H2("Feature Distribution Analysis", style=header_h2_style),
                                html.Div([
                                    html.Div([
                                        html.H3("Variable Distribution by Segment", style = subheader_style),
                                        dcc.Dropdown(id='feature-dropdown', placeholder="Select a feature to display", style = dropdown_style),
                                        html.Div(id="feature-histograms"),
                                    ], style={'width': '49%', 'display': 'inline-block'}),
                                    html.Div([
                                        html.H3("Average Feature Values by Segment", style = subheader_style),
                                        dcc.Dropdown(id='numerical-feature-multiselect', multi=True, placeholder="Select numerical features", style = dropdown_style),
                                        html.Div(id='mean-heatmap'),
                                    ], style={'width': '49%', 'display': 'inline-block', 'float': 'right'}),
                                ]),

                                html.Div(id="message-div", style={"color": "black", "font-weight": "bold"})
                            ], style={'width': '100%', 'padding': '20px'})
                        ])
                ])
            ],style={
        'display': 'flex',  # Makes the tabs appear in a row
        'justify-content': 'flex-start',  # Align tabs to the start (left)
        'margin': '0px 5px',  # Small margin around the tabs
        'border': 'none'  # Bottom border for the tabs container
    },)
        ], style=right_panel_style)  # Right panel styling
    ], style=main_style)  # Flexbox for main container
])

# Callback to synchronize the tabs in both the left and right panels
@app.callback(
    [Output('left-panel-tabs', 'value'), Output('right-panel-tabs', 'value')],
    [Input('left-panel-tabs', 'value'), Input('right-panel-tabs', 'value')]
)
def sync_tabs(left_tab, right_tab):
    ctx = dash.callback_context
    if not ctx.triggered:
        return dash.no_update, dash.no_update  # No updates if nothing is triggered
    triggered_id = ctx.triggered[0]['prop_id'].split('.')[0]
    if triggered_id == 'left-panel-tabs':
        return left_tab, left_tab
    else:
        return right_tab, right_tab



# Callback to update the mini cards based on the search input
@app.callback(
    Output('card-container', 'children'),
    [Input('search-bar', 'value')]
)
def update_cards(search_value):
    # Filter metadata based on the search input
    metadata_display = metadata_active[['session_name', 'original_dataset', 'username', 'datetime', 'method', 'description', 'selected_features', 'specifications', 'cluster_name_remap', 'segmentation_version']]
    
    if search_value:
        metadata_display = metadata_display[metadata_display['session_name'].str.contains(search_value, case=False, na=False)]

    # Generate the cards with tooltips
    cards = [
        html.Div([
            html.Button(
                id={'type': 'card-button', 'index': i},
                children=html.Div([
                    html.H4(row['session_name'], style = card_margin),
                    html.P(f"Dataset: {row['original_dataset']}", style = card_margin),
                    html.P(f"User: {row['username']}", style = card_margin),
                    html.P(f"Date: {row['datetime']}", style = card_margin),
                    html.P(f"Method: {row['method']}", style = card_margin),
                ]),
                style=card_style,
                className='hover-effect'
            ),
            dbc.Tooltip(
                #f"Description: {row['description']}<br>Features: {row['selected_features']}",
                html.Div([
                f"Description: {row['description']}",
                html.Br(),
                f"Features: {row['selected_features']}",
                html.Br(),
                f"Specifications: {row['specifications']}"   
            ]),
                target={'type': 'card-button', 'index': i}, 
                placement="right",
                style = tooltip_style
            )
        ], className='mini-card')
        for i, row in metadata_display.iterrows()
    ]
    return cards


@app.callback(
    Output({'type': 'card-button', 'index': ALL}, 'style'),
    [Input({'type': 'card-button', 'index': ALL}, 'n_clicks')]
)
def update_card_selection(n_clicks):
    # Default all cards to unselected style
    card_styles = [card_style] * len(n_clicks)

    # Get the triggered context
    ctx = dash.callback_context

    if not ctx.triggered or all(click is None for click in n_clicks):
        return card_styles

    # Extract the clicked card's index from ctx.triggered
    clicked_id = ctx.triggered[0]['prop_id'].split('.')[0]
    clicked_index = int(eval(clicked_id)['index'])  # Safely extract the index

    # Update the clicked card's style to selected
    card_styles[clicked_index] = selected_card_style

    return card_styles

@app.callback(
    [Output('selected-session-name', 'children'),
     Output('file-content-container', 'children'),
     Output('update-button', 'style'),  # Control visibility of the update button
     Output('tooltip-update-button', 'style')],  # Control visibility of the question mark icon
    [Input({'type': 'card-button', 'index': dash.dependencies.ALL}, 'n_clicks')]
)
def select_session(n_clicks):
    if not n_clicks or all(click is None for click in n_clicks):
        # No session selected yet, hide both the button and the question mark
        return '', html.Div("No data selected yet.", style={'color': 'black', 'margin-top': '14px', 'fontsize':'12px'}), {'display': 'none'}, {'display': 'none'}

    # Determine which card button was clicked
    clicked_index = dash.callback_context.triggered_id['index']
    session_name = metadata_active.iloc[clicked_index]['session_name']
    
    # Retrieve data for the selected session
    columns, data = display_file_content(session_name)
    
    if not data:
        # No data available for the selected session, hide both the button and the question mark
        return f'Selected Session: {session_name}', html.Div("No data available for the selected session.", style={'margin-top': '10px'}), {'display': 'none'}, {'display': 'none'}
    
    # Show the table if data is available and make both the button and the question mark visible
    table_component = dash_table.DataTable(
        id='file-content-table',
        columns=columns,
        data=data,
        filter_action='native',
        sort_action='native',
        page_size=5,
        **table_styles
    )
    
    return f'Selected Session: {session_name}', table_component, {**button_style, 'display': 'block'}, {**question_mark_style,'display': 'inline-block'}  


@app.callback(
    [Output('table-container', 'style'),  
     Output('no-data-message', 'children'),  
     Output('no-data-message', 'style'),  
     Output('output_table', 'columns'),  
     Output('output_table', 'data'),  
     Output('update-message', 'children'),  
     Output('session-name-store', 'data'),  
     Output('selected-features-store', 'data'),  
     Output('feature-dropdown', 'options'),  
     Output('version-store', 'data'), 
     Output('original-dataset-store', 'data'), 
     Output('method-store', 'data'),  
     Output('specifications-store', 'data'),  
     Output('remap-dict-store', 'data'), 
     Output('importance-histogram', 'figure'), 
     Output('sankey-diagram', 'figure'), 
     Output('full-data-store', 'data')],  
    [Input('update-button', 'n_clicks')],
    [State('selected-session-name', 'children'),
     State('file-content-table', 'data')]
)
def update_segmentation(n_clicks, selected_session_text, file_content_data):
    # Default hidden and visible styles
    empty_figure_sankey = {
            'data': [],
            'layout': {
                'xaxis': {'visible': False},
                'yaxis': {'visible': False},
                'annotations': [{'text': " ", 'xref': "paper", 'yref': "paper", 'showarrow': False, 'font': {'size': 12}}]
            }
        }
    empty_figure_var = {
            'data': [],
            'layout': {
                'xaxis': {'visible': False},
                'yaxis': {'visible': False},
                'annotations': [{'text': "No data selected yet.", 'xref': "paper", 'yref': "paper", 'showarrow': False, 'font': {'size': 12}}]
            }
        }

    if n_clicks is None or selected_session_text == '':
        # Return empty figures and message when no update has happened
        message_div = " "
        
        return hidden_style, message_div, visible_style, [], [], '', None, None, [], None, None, None, None, {}, empty_figure_var, empty_figure_sankey, None

    session_name = selected_session_text.replace('Selected Session: ', '')
    metadata_session = metadata_active[(metadata_active['session_name'] == session_name)]
    
    original_dataset_name = metadata_session['original_dataset'].iloc[0]
    selected_features = ast.literal_eval(metadata_session['selected_features'].iloc[0])
    segmentation_method = metadata_session['method'].iloc[0]
    specifications = ast.literal_eval(metadata_session['specifications'].iloc[0])
    cluster_remap_dict = ast.literal_eval(metadata_session['cluster_name_remap'].iloc[0])
    segmentation_version = metadata_session['segmentation_version'].iloc[0]

    dataset_name = original_dataset_name
    if dataset_name not in [ds.name for ds in project.list_datasets()]:
        error_message_data = f"Error: dataset '{dataset_name}' does not exist in the Dataiku flow. Please check the name of existing datasets to align with the selected sessions."
        return hidden_style,error_message_data, visible_style, [], [], '', None, None, [], None, None, None, None, {}, empty_figure_var, empty_figure_sankey, None

    # Proceed to load the dataset since it exists
    original_dataset = dataiku.Dataset(dataset_name)
    original_dataset_df = original_dataset.get_dataframe()
    
    if all(col in original_dataset_df.columns for col in selected_features):
        if not bool(specifications):
            filtered_df = original_dataset_df.copy()
        else:
            filtered_df = apply_filters(original_dataset_df, specifications)
        
        if filtered_df.empty:
            return hidden_style, "Filtered data is empty. Please check the original input dataset and try again.", visible_style, [], [], None, [], None, None, None, None, {}, empty_figure_var, empty_figure_sankey, None

        cluster_column_name = 'cluster'
        
        if file_content_data:
            df_file_content = pd.DataFrame(file_content_data)
            
        if segmentation_method == "kmeans":
            model_folder_id = output_model_folder_var
            df_results, preprocessor = apply_existing_model(cluster_column_name, filtered_df, model_folder_id, session_name, cluster_remap_dict)
            feature_importance_df = feature_importance_rfclassifier(df_results, selected_features, preprocessor, cluster_column_name)
        else:
            df_results, weights, warning_message = apply_existing_bounds(session_name, selected_features, df_file_content, filtered_df, cluster_remap_dict)
            if warning_message:
                return hidden_style, warning_message, visible_style, [], [], None, [], None, None, None, None, {}, {}, {}, None
            importance_dict = weights
            feature_importance_df = pd.DataFrame(list(importance_dict.items()), columns=['Feature', 'Importance']).sort_values(by='Importance', ascending=False)
        
        
        if file_content_data:
            df_file_content = df_file_content[['account_id', 'cluster']].rename(columns={'cluster': 'old_cluster'})
            df_results = pd.merge(df_results, df_file_content, on='account_id', how='outer')
        
        cluster_id_columns = ['account_id', 'old_cluster', 'cluster']
        df_results_filtered = df_results[cluster_id_columns]
        df_results_changed = df_results_filtered[df_results_filtered['old_cluster'] != df_results_filtered['cluster']]        
        # Get numerical columns
        numerical_columns = df_results.select_dtypes(include=['number']).columns.tolist()
        # Get categorical columns
        categorical_columns = df_results.select_dtypes(include=['object', 'category']).columns.tolist()
        columns_list = numerical_columns + categorical_columns

        # Remove the specified columns from the list
        columns_list = [col for col in columns_list if col not in cluster_id_columns]
        
        feature_options = [{"label": feature, "value": feature} for feature in columns_list]
        
        feature_importance_df = feature_importance_df.sort_values('Importance', ascending=False).head(10)
        histogram_fig = histogram_figure(feature_importance_df, "Importance", "Feature", {'Importance': 'Importance Score', 'Feature': 'Features'})

        columns = [{"name": i, "id": i} for i in df_results_changed.columns]
        data = df_results_changed.to_dict('records')
        
        # Check if there are any differences between old and new cluster
        if df_results_changed.empty:
            sankey_fig = empty_figure_sankey 
            update_message = f"Segmentation results for {session_name} are the same."
            table_style = hidden_style
        else:
            sankey_fig = create_sankey(df_results_filtered, 'old_cluster', 'cluster')
            update_message = " "
            table_style = visible_style

        #sankey_fig, update_message, table_style = create_sankey(df_results_filtered, 'old_cluster', 'cluster', session_name)
        df_results_new = df_results.drop(columns=['old_cluster'])
        # Reorder columns by placing 'cluster' after 'account_id'
        result_cols = df_results_new.columns.tolist()
        result_cols.insert(result_cols.index('account_id') + 1, result_cols.pop(result_cols.index('cluster')))
        df_results_new = df_results_new[result_cols]
        full_data_store = df_results_new.to_dict('records')

        
        return table_style, "", hidden_style, columns, data, update_message, session_name, selected_features, feature_options, segmentation_version, original_dataset_name, segmentation_method, specifications, cluster_remap_dict, histogram_fig, sankey_fig, full_data_store

    else:
        return hidden_style, f"Not all columns used for the original segmentation exist in the updated dataset. Check '{selected_features}'", visible_style, [], [], None, [], None, None, None, None, {}, empty_figure_var, empty_figure_sankey, None

@app.callback(
    Output("feature-histograms", "children"),
    [Input("feature-dropdown", "value")],
    [State("full-data-store", "data")]
)
def update_histograms_output(selected_feature, table_data):
    return update_mixed_graph(selected_feature, table_data)
 
@app.callback(
    Output('cluster-histogram', 'figure'),
    [Input('full-data-store', 'data')]
)
def update_cluster_pie_chart_output(table_data):
    if not table_data:
        # Return an empty figure with no axes if no data is available
        return {
            'data': [],
            'layout': {
                'xaxis': {'visible': False},
                'yaxis': {'visible': False},
                'annotations': [{'text': "No data selected yet.", 'xref': "paper", 'yref': "paper", 'showarrow': False, 'font': {'size': 12}}]
            }
        }

    return update_cluster_pie_chart(table_data)


@app.callback(
    Output('numerical-feature-multiselect', 'options'),
    [Input('full-data-store', 'data')]
)
def update_numerical_feature_multiselect_output(table_data):
    return update_numerical_feature_multiselect(table_data)

@app.callback(
    Output('numerical-feature-multiselect', 'value'),
    Input('numerical-feature-multiselect', 'value'),
    State('numerical-feature-multiselect', 'options')
)
def handle_select_all_numerical_feature_multiselect(selected_values, available_options):
    # If "Select All" is selected, return all available options (excluding "Select All" itself)
    # Check if selected_values or available_options is None
    if selected_values is None or available_options is None:
        return []
    if "select_all" in selected_values:
        return [option['value'] for option in available_options if option['value'] != 'select_all']
    return selected_values

@app.callback(
    Output('mean-heatmap', 'children'),
    [Input('numerical-feature-multiselect', 'value')],
    [State('full-data-store', 'data')]  
)
def update_heatmap(selected_numerical_features, table_data):
    if table_data and selected_numerical_features:
        df = pd.DataFrame(table_data)
        return create_average_heatmap(df, selected_numerical_features, cluster_column_name='cluster')
    return {}  

@app.callback(
    Output("download-dataframe-csv", "data"),
    Input("export-icon", "n_clicks"),
    [State('session-name-store', 'data'),
    State("full-data-store", "data")],
    prevent_initial_call=True,
)
def export_data_as_csv(n_clicks, session_name, output_data):
    if n_clicks > 0 and output_data is not None:
        df = pd.DataFrame(output_data)
        return dcc.send_data_frame(df.to_csv, session_name + "_results.csv")
    return None

@app.callback(
    Output("description-dictionary", "data"),
    Input("metadata-description", "value")
)
def store_description(description):
    if description:
        return {"description": description}
    return {}

@app.callback(
    Output('save-output', 'children'),  # Output for displaying error or success messages
    [Input('save-icon', 'n_clicks')],
    [State("full-data-store", "data"),
     State('remap-dict-store', 'data'),  
     State('session-name-store', 'data'),
     State('version-store', 'data'),
     State('original-dataset-store', 'data'),
     State('method-store', 'data'),
     State('description-dictionary', 'data'),
     State('specifications-store', 'data'),
     State('selected-features-store', 'data')],
    prevent_initial_call=True
)
def save_data(save_clicks, output_data, cluster_remap_dict, session_name, segmentation_version, original_dataset_name, segmentation_method, description_dict, specifications, selected_features):    
    triggered = ctx.triggered_id
    df = pd.DataFrame(output_data)
    
    # Check if the callback was triggered by the 'save-icon'
    if triggered == 'save-icon':
        
        # Check if the description is provided
        description = description_dict.get("description", "") if description_dict else ""
        if not description:
            return "Error: You must provide a description before saving the data."
        
        # Proceed with saving if all necessary information is available
        if output_data and session_name and segmentation_version is not None:
            updated_version = segmentation_version + 1
            status = 'active'
            metadata_record = create_metadata_record(
                session_name, updated_version, status, original_dataset_name, segmentation_method, description, specifications, selected_features, cluster_remap_dict
            )

            return save_updated_data_to_dataiku(session_name, updated_version, df, metadata_record)
        else:
            return "Failed to save data. Please check if all necessary data is available."

    return ""
