import dash
from dash import dcc, html, dash_table, Input, Output, State
import pandas as pd
import dataiku
from main_functions import *
from layout_components_style import get_main_style, get_left_panel_style, get_right_panel_style, get_table_styles, get_card_style,get_button_style, get_enter_box_style,  get_header_h2_style, get_subheader_style, get_tooltip_style, get_question_mark_style
import dash_bootstrap_components as dbc

card_style = get_card_style()
table_style = get_table_styles()
button_style=get_button_style()
enter_box_style = get_enter_box_style()
# Define display styles
hidden_style = {'display': 'none'}
visible_style = {'display': 'block'}
tooltip_style = get_tooltip_style() 
question_mark_style = get_question_mark_style()
header_h2_style = get_header_h2_style()
subheader_style = get_subheader_style()
main_style = get_main_style()
left_panel_style = get_left_panel_style()
right_panel_style = get_right_panel_style()
table_styles = get_table_styles()
card_margin =  {'margin': '3px 0', 'padding': '0'}
    

# Read and process the metadata dataset
metadata_df['version_name'] = metadata_df['session_name'] + "_" + metadata_df["segmentation_version"].astype(str)
metadata_first_column = metadata_df.pop('version_name')
metadata_df.insert(0, 'version_name', metadata_first_column)
metadata_df = metadata_df.drop(columns=['session_name', 'segmentation_version'])
# Read and merge files
folder_path = versions_folder.get_path()
compare_seg_folder = dataiku.Folder("LvSKpnPJ")

# Define layout
app.layout = html.Div([
    # Title
    #html.H1('Compare Segmentation Sessions', style={'text-align': 'center', 'fontSize': '24px', 'padding': '10px'}),

    # Main container for left and right panels
    html.Div([
        # Left panel for mini cards
        html.Div([
            html.H2(['Select Sessions',
                                html.Span('?', id='tooltip-select-sessions', style=question_mark_style)
                            ], style=header_h2_style),
                            dbc.Tooltip(
                                "Choose multiple segmentation sessions to join side-by-side on the account_id key column and display on the right side. This method right joins the data on the first dataset you select. If the data are not updated when selecting multiple datasets it implies that there are no common accounts.",
                                target="tooltip-select-sessions",  
                                placement="right",
                                style = tooltip_style
                            ),
            dcc.Input(
                id='search-bar',
                type='text',
                placeholder='Search sessions',
                style=enter_box_style
            ),
            html.Div(
                id='card-container',
                className='card-container',
                style={
                                'height': '900px',
                                'overflow-y': 'scroll',
                                'border': '1px solid #ddd',
                                'padding': '10px',
                                'background-color': 'white'
                            }
            )
        ],  style=left_panel_style),
        
        # Right panel for displaying merged data
        html.Div([
            # Refactored layout with hidden table and message display
                html.Div([
                    html.Div(id="joined-data-container", children=[
                        html.H2('Joined Data', style=header_h2_style),
                        dash_table.DataTable(
                            id='merged-data-table', 
                            columns=[],
                            data=[],
                            filter_action='native',
                            page_size=16,
                            page_action='native',  # Enables pagination
                            page_current=0, 
                            **table_styles
                        ),
                         html.H2(['Data Processing',
                                html.Span('?', id='tooltip-data-process-comp', style=question_mark_style)
                            ], style=header_h2_style),
                            dbc.Tooltip(
                                "Give a session name to the above dataset and export it back to the dataiku flow in csv format within compare_sessions folder. Within than folder you can select the file and create a dataset on the flow to further explore in the Dataiku Dashboards.",
                                target="tooltip-data-process-comp",  
                                placement="right",
                                style = tooltip_style
                            ),
                    ], style=hidden_style), 
                    
                    
                    html.Div("Select sessions to compare.", id="select-message", style=visible_style)  # Initially visible
                ]),
            
            
            # File name input, export button, and success message
            html.Div([
                dcc.Input(
                    id='file-name-input',
                    type='text',
                    placeholder='Enter a file name',
                    style={**enter_box_style, 'margin-top': '20px', 'width': '100%', 'display': 'none'}
                ),
                html.Button('Export the data to Compare Sessions Folder\nand further explore insights in Dataiku Dashboards', 
                            id='export-button', 
                            style=button_style),
                html.Div(id='success-message', style={'margin-top': '20px', 'display': 'none'})
            ])
        ], style=right_panel_style)  # Right Panel styling
    ],  style=main_style)
])


@app.callback(
    Output('card-container', 'children'),
    Input('search-bar', 'value')
)
def update_cards(search_value):
    # Filter the metadata based on the search input
    if search_value:
        filtered_df = metadata_df[metadata_df['version_name'].str.contains(search_value, case=False)]
    else:
        filtered_df = metadata_df

    # Generate the mini cards based on the filtered data
    cards = [
        html.Div(
            children=[
                dcc.Checklist(
                    options=[{'label': '', 'value': row['version_name']}],
                    value=[],
                    id={'type': 'session-card', 'index': idx},
                    style={'margin-bottom': '5px'}  # Add spacing between the checkbox and the next element
                ),
                html.Label(f"{row['version_name']} ({row['status']})", style={'font-weight': 'bold', 'margin': '3px 0', 'padding': '0'}),
                html.P(f"User: {row['username']}", style=card_margin),
                html.P(f"Date: {row['datetime']}", style=card_margin),
                html.P(f"Method: {row['method']}", style=card_margin),
            ],
            style=card_style,
            className='hover-effect',
            id={'type': 'card-div', 'index': idx}  
        )
        for idx, row in filtered_df.iterrows()
    ]

    # Add tooltips dynamically for each card
    tooltips = [
        dbc.Tooltip(
            f"Description: {row['description']}",
            target={'type': 'card-div', 'index': idx},  
            placement="right",
            style = tooltip_style
        )
        for idx, row in filtered_df.iterrows()
    ]

    return cards + tooltips


@app.callback(
    [Output('joined-data-container', 'style'),  # Controls visibility of the table section
     Output('select-message', 'style'),  # Controls visibility of the message
     Output('merged-data-table', 'data'),
     Output('merged-data-table', 'columns'),
     Output('file-name-input', 'style'),
     Output('export-button', 'style')],
    Input({'type': 'session-card', 'index': dash.dependencies.ALL}, 'value')
)
def update_table_and_visibility(selected_sessions_values):
    selected_sessions = [item for sublist in selected_sessions_values for item in sublist]
    
    if not selected_sessions:
        # No sessions selected: Hide the table and show the message
        return hidden_style, visible_style, [], [], hidden_style, hidden_style

    # Sessions selected: Show the table and hide the message
    selected_sessions_df = metadata_df[metadata_df['version_name'].isin(selected_sessions)]
    sessions_list = selected_sessions_df['version_name'].tolist()

    # Process the data as per your logic
    sessions_list_paths = construct_file_paths(sessions_list)
    merged_data = read_and_merge_files(versions_folder, sessions_list_paths, sessions_list)

    # Prepare data and columns for dash_table
    data = merged_data.to_dict('records')
    columns = [{"name": col, "id": col} for col in merged_data.columns]

    # Show the table and input button, hide the message
    return visible_style, hidden_style, data, columns, {**enter_box_style, **visible_style}, {**button_style, **visible_style}



# Callback to handle export and success message
@app.callback(
    Output('file-name-input', 'value'),
    Output('success-message', 'children'),
    Output('success-message', 'style'),
    Input('export-button', 'n_clicks'),
    State('merged-data-table', 'data'),
    State('file-name-input', 'value')
)
def export_data(n_clicks, data, file_name):
    if n_clicks and data and file_name:
        df = pd.DataFrame(data)
        compare_filename = f"{file_name}.csv"
        compare_seg_folder.upload_data(compare_filename, df.to_csv(index=False).encode("utf-8"))
        return '', 'Data have been successfully saved in the compare sessions folder!', {'color': 'black', 'margin-top': '10px', 'display': 'block'}
    return file_name, '', {'display': 'none'}
